/* Modified by Pascal Lacroix <placr@hotmail.com> for LittleOS */

/* 
 * Copyright (c) 1995 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 *
 *      Author: Bryan Ford, University of Utah CSL
 */

#ifndef __LMM_H

#define __LMM_H

#if !defined(DEBUG) && !defined(NDEBUG)
#define NDEBUG
#endif

#include <assert.h>

#define PAGE_SIZE 4096
#define PAGE_SHIFT 12

#define phystokv(addr) (addr)
#define kvtophys(addr) (addr)

/* this is needed for Mach sources */
#define printf printk

typedef struct lmm
{
    struct lmm_region *regions;
}
lmm_t;

#define LMM_INITIALIZER { 0 }

struct lmm_region
{
    struct lmm_region *next;

    /* List of free nodes in this region.  */
    struct lmm_node *nodes;

    /* Attributes of this memory.  */
    unsigned int flags;

    /* Allocation priority of this region with respect to other regions.  */
    int pri;

    /* Total size of the region including this header structure.  */
    unsigned int size;

    /* Current amount of free space in this region in bytes.  */
    unsigned int free;
};

struct lmm_node
{
    struct lmm_node *next;
    unsigned int size;
};

void lmm_init(lmm_t * lmm);
void lmm_add(lmm_t * lmm, unsigned int addr, unsigned int size,
	     unsigned int flags, int pri);
void *lmm_alloc(lmm_t * lmm, unsigned int size, unsigned int flags);
void *lmm_alloc_aligned(lmm_t * lmm, unsigned int size, unsigned int flags,
			int align_bits, unsigned int align_ofs);
void *lmm_alloc_page(lmm_t * lmm, unsigned int flags);
void *lmm_alloc_gen(lmm_t * lmm, unsigned int size, unsigned int flags,
		    int align_bits, unsigned int align_ofs,
		    unsigned int bounds_min, unsigned int bounds_max);
unsigned int lmm_avail(lmm_t * lmm, unsigned int flags);
void lmm_find_free(lmm_t * lmm, unsigned int *inout_addr,
		   unsigned int *out_size, unsigned int *out_flags);
void lmm_free(lmm_t * lmm, void *block, unsigned int size);

/* Only available if debugging turned on.  */
void lmm_dump(lmm_t * lmm);

void lmm_remove_free(lmm_t * lmm, unsigned int rstart, unsigned int rsize);

#define ALIGN_SIZE	sizeof(struct lmm_node)
#define ALIGN_MASK	(ALIGN_SIZE - 1)

#endif
