/*
 * LittleOS - http://come.to/littleos/
 * Copyright (C) 1998-2000 Lacroix Pascal <placr@hotmail.com>
 * All rights reserved.
 *
 * LittleOS is free software -- also known as "open source" -- you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License, version 2, as published by the Free Software Foundation,
 * and found in the file `COPYING'.  This license basically allows free use,
 * modification, and redistribution, as long as the source to LittleOS and
 * any code linked against it is made freely available.  (The license in
 * `COPYING' is authoritative, not this interpretation of it.)
 *
 * LittleOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with LittleOS; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place #330, Boston, MA 02111-1307, USA, or look
 * at http://www.fsf.org/copyleft/gpl.html.
 *
 */

#ifndef _SYS_PAGING_H
#define _SYS_PAGING_H

/* initialize the paging system: create page tables and set CR3 */
void init_paging(void);

/* some useful constants */
#define PAGE_PRESENT   1
#define PAGE_READWRITE 2
#define PAGE_USER      4
#define PAGE_RW PAGE_READWRITE

#define PAGE_SIZE 4096
#define PAGE_SHIFT 12

#define PD_SHIFT 22		/* page descriptor shift */
#define PD_MASK	0x3ff		/* mask for page descriptor index */
#define PT_SHIFT 12		/* page table shift */
#define PT_MASK	0x3ff		/* mask for page table index */

/* a helpful macro to align an address to PAGE_SIZE */
#define ALIGN_PAGE(addr) ( ((addr) + PAGE_SIZE-1) & ~(PAGE_SIZE-1) )

#define pdir_find_pde(pdir_pa, la)					\
	(&((unsigned long*)(pdir_pa))[((la)>>PD_SHIFT)&PD_MASK])
#define ptab_find_pte(ptab_pa, la)					\
	(&((unsigned long*)(ptab_pa))[((la)>>PT_SHIFT)&PT_MASK])

bool vm_map_page(unsigned long, unsigned long);
bool vm_map_range(unsigned long, unsigned long, unsigned long);
bool vm_prot_page(unsigned long, long);
bool vm_unmap_page(unsigned long);
bool vm_remap_page(unsigned long, long);

#endif
