[SEGMENT .text]
[BITS 32]

extern int_no
extern handle_int

//; this is the default routine (for the interrupt table) used in intr.c
[GLOBAL _dummy_int]
[GLOBAL dummy_int]

align 4
_dummy_int:
dummy_int:
          mov al, 0x20
          out 0x20, al
          out 0xA0, al
          iret

//;----------------------------------------------------------------------------
[GLOBAL ext_asm_ldt]
ext_asm_ldt:
    mov ax, 0x38
    lldt ax
    mov eax, dword 9
    mov ds, ax
    mov es, ax
    ret


//; macros to save/restore all registers before/after a trap-handler

%macro enter_int 0
	push eax
	push ecx
	push edx
	push ebx
	push ebp
	push esi
	push edi
	push ds
	push es
        push fs
        push gs
	mov ax, ss
	mov ds, ax
	mov es, ax
	cld
        mov eax, esp
        push eax
%endmacro

align 4
__leave_int:
        add esp, 4
        pop gs
        pop fs
	pop es
	pop ds
	pop edi
	pop esi
	pop ebp
	pop ebx
	pop edx
	pop ecx
	pop eax
	add esp, 4
	iret

%macro leave_int 0
       jmp __leave_int
%endmacro

%macro make_entry 2
[GLOBAL __int_%1]
align 4
__int_%1:
	push dword %2
	enter_int
%endmacro

%macro make_entry_with_error 2
[GLOBAL __int_%1]
align 4
__int_%1:
	enter_int
%endmacro

//; interrupt routine with error code (exception)
%macro interrupt_with_error 2
	make_entry_with_error %1,%2
        mov dword [int_no], %2
	call handle_int
	leave_int
%endmacro

//; interrupt routine without error code (exception)
%macro interrupt 2
	make_entry %1, 0
        mov dword [int_no], %2
	call handle_int
	leave_int
%endmacro

//; this must be preprocced by cpp (gcc -E)          

#define INTHANDLER(name, no) interrupt name, no
#define INTERRHANDLER(name, no) interrupt_with_error name, no
#define IRQAHANDLER(name, no) interrupt name, no
#define IRQBHANDLER(name, no) interrupt name, no

#include "interrupts.lst"
