/*
 * LittleOS - http://come.to/littleos/
 * Copyright (C) 1998-2000 Lacroix Pascal <placr@hotmail.com>
 * All rights reserved.
 *
 * LittleOS is free software -- also known as "open source" -- you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License, version 2, as published by the Free Software Foundation,
 * and found in the file `COPYING'.  This license basically allows free use,
 * modification, and redistribution, as long as the source to LittleOS and
 * any code linked against it is made freely available.  (The license in
 * `COPYING' is authoritative, not this interpretation of it.)
 *
 * LittleOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with LittleOS; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place #330, Boston, MA 02111-1307, USA, or look
 * at http://www.fsf.org/copyleft/gpl.html.
 *
 */

#include <types.h>
#include <string.h>
#include <sys/intr.h>
#include <sys/multiboot.h>
#include <sys/kernel.h>
#include <sys/memory.h>
#include <sys/tasks.h>
#include <sys/paging.h>

/* the structure passed by the boot loader is copied here */
struct multiboot_info boot_info;

/* kernel command line used to pass some parameters */
char kernel_cmdline[255];

extern void init_pci();

/* this routine is called from entry.asm */
void
c_boot_entry(struct multiboot_info *mb_info, long bootloader_signature)
{
    if (bootloader_signature != MULTIBOOT_VALID)
    {
	panic("bootloader_flag is not valid: %08X\n", bootloader_signature);
    }

    /* before all, we copy the multiboot_info structure (before
       erasing GRUB's memory) */
    memcpy((void *) &boot_info, mb_info, sizeof(struct multiboot_info));

    strncpy(kernel_cmdline, (char *) boot_info.cmdline, 255);

    /* print system version */
    {
	struct utsname name;

	uname(&name);
	printk("\27\2%s\27\7 \27\1%s.%s %s\27\7\n", name.sysname, name.version, name.release,
	       name.machine);
    }

    detect_cpu();

    /* now we must initialize memory and multitasking */
    init_memory();

    init_paging();		/*TODO: really TEST THESE ROUTINES */

    init_pci();			/* TODO: declare prototype */

    init_tasking();		/* nothing yet */

    printk("\n\27\15Welcome to LittleOS!\n\27\7");
    printk("All seem to work fine. That's all for the moment.\n");
    printk("You can now reboot.");

    while (1);
}
