/*
 * LittleOS - http://come.to/littleos/
 * Copyright (C) 1998-2000 Lacroix Pascal <placr@hotmail.com>
 * All rights reserved.
 *
 * LittleOS is free software -- also known as "open source" -- you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License, version 2, as published by the Free Software Foundation,
 * and found in the file `COPYING'.  This license basically allows free use,
 * modification, and redistribution, as long as the source to LittleOS and
 * any code linked against it is made freely available.  (The license in
 * `COPYING' is authoritative, not this interpretation of it.)
 *
 * LittleOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with LittleOS; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place #330, Boston, MA 02111-1307, USA, or look
 * at http://www.fsf.org/copyleft/gpl.html.
 *
 */

#include <types.h>
#include <sys/kernel.h>
#include <sys/multiboot.h>
#include <sys/memory.h>
#include <i386/phys_mem.h>
#include <malloc.h>

/* macro useful to align to PAGE_SIZE (=4096) */
#define ALIGN_PAGE(addr) ( ((addr) + PAGE_SIZE-1) & ~(PAGE_SIZE-1) )

/* used for the modules */
extern struct multiboot_info boot_info;

unsigned long total_memory, end_of_kernel, start_of_kernel;

extern long end;
extern long start;

void
init_memory(void)
{
    start_of_kernel = (long) &start;
    end_of_kernel = (long) &end;
    if (boot_info.flags & MB_INFO_MODS)		/* are there some modules ? */
    {
	int i;
	struct mod_list *ml = (struct mod_list *) boot_info.mods_addr;

	for (i = 0; i < boot_info.mods_count; i++)
	    if (end_of_kernel < ml[i].mod_end)
		end_of_kernel = ml[i].mod_end;
	/* yes, we must adjust end_of_kernel */
    }

    /* align to PAGE_SIZE (=4096) */
    end_of_kernel = ALIGN_PAGE(end_of_kernel);

    total_memory = (boot_info.mem_upper + 1024) * 1024;

    lmm_init(&malloc_lmm);

    if (boot_info.flags & MB_INFO_MEM_MAP) /* is there a memory map ? */
    {
	struct AddrRangeDesc *ard = (struct AddrRangeDesc *) boot_info.mmap_addr;
	int len = boot_info.mmap_length;

	while (len > 0)
	{
	    if (ard->Type == MB_ARD_MEMORY)
	    {
		/* free memory, add it to the list */
		if (ard->BaseAddrLow == 0)
		{
		    ard->BaseAddrLow += PAGE_SIZE;
		    ard->LengthLow -= PAGE_SIZE;
		}
		phys_mem_add(ard->BaseAddrLow, ard->LengthLow);

#ifdef DEBUG
		printk("adding free memory: start= %p, length= %p\n", ard->BaseAddrLow, ard->LengthLow);
#endif
		assert(ard->BaseAddrHigh == 0);
		assert(ard->LengthHigh == 0);
	    }
	    ard->size += 4;
	    len -= ard->size;
	    (char *) ard += ard->size;
	}
    }
    else
    {	/* no memory map from loader, so we must add "manually" free memory */
	phys_mem_add(PAGE_SIZE, boot_info.mem_lower * 1024 - PAGE_SIZE);
	phys_mem_add(0x100000, boot_info.mem_upper * 1024);
    }

    /* now we must set as "reserved" the memory used by the kernel
     * and the modules */
    lmm_remove_free(&malloc_lmm, start_of_kernel, end_of_kernel - start_of_kernel);
    lmm_remove_free(&malloc_lmm, 0xA0000, (0x100000 - 1) - 0xA0000);

    info("memory: total= %d K, free= %d K, kernel= %d K\n",
	 total_memory / 1024, lmm_avail(&malloc_lmm, 0) / 1024,
	 (end_of_kernel - start_of_kernel) / 1024);

}
