/* Modified by Pascal Lacroix <placr@hotmail.com> for LittleOS */
/* 
 * Copyright (c) 1995 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 *
 *      Author: Bryan Ford, University of Utah CSL
 */

#include <sys/lmm.h>

void
lmm_find_free(lmm_t * lmm, unsigned int *inout_addr,
	      unsigned int *out_size, unsigned int *out_flags)
{
    struct lmm_region *reg;
    unsigned int start_addr = (*inout_addr + ALIGN_MASK) & ~ALIGN_MASK;
    unsigned int lowest_addr = (unsigned int) -1;
    unsigned int lowest_size = 0;
    unsigned lowest_flags = 0;

    for (reg = lmm->regions; reg; reg = reg->next)
    {
	struct lmm_node *node;

	if ((reg->nodes == 0)
	    || ((unsigned int) reg + reg->size <= start_addr)
	    || ((unsigned int) reg > lowest_addr))
	    continue;

	for (node = reg->nodes; node; node = node->next)
	{
	    assert((unsigned int) node > (unsigned int) reg);
	    assert((unsigned int) node < (unsigned int) reg + reg->size);

	    if ((unsigned int) node >= lowest_addr)
		break;
	    if ((unsigned int) node + node->size > start_addr)
	    {
		if ((unsigned int) node > start_addr)
		{
		    lowest_addr = (unsigned int) node;
		    lowest_size = node->size;
		}
		else
		{
		    lowest_addr = start_addr;
		    lowest_size = node->size
			- (lowest_addr - (unsigned int) node);
		}
		lowest_flags = reg->flags;
		break;
	    }
	}
    }

    *inout_addr = lowest_addr;
    *out_size = lowest_size;
    *out_flags = lowest_flags;
}
