
/* Modified by Pascal Lacroix <placr@hotmail.com> for LittleOS */
/* 
 * Copyright (c) 1995 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 *
 *      Author: Bryan Ford, University of Utah CSL
 */

#include <sys/lmm.h>

void
lmm_free(lmm_t * lmm, void *block, unsigned int size)
{
    struct lmm_region *reg;
    struct lmm_node *node = (struct lmm_node *) ((unsigned int) block & ~ALIGN_MASK);
    struct lmm_node *prevnode, *nextnode;

    size = (((unsigned int) block & ALIGN_MASK) + size + ALIGN_MASK) & ~ALIGN_MASK;

    /* First find the region to add this block to.  */
    for (reg = lmm->regions;; reg = reg->next)
    {
	assert(reg != 0);
	assert((unsigned int) reg->nodes >= (unsigned int) (reg + 1));
	assert(reg->free <= reg->size - sizeof(struct lmm_region));

	if (((unsigned int) node >= (unsigned int) reg)
	    && ((unsigned int) node < (unsigned int) reg + reg->size))
	    break;
    }

    /* Record the newly freed space in the region's free space counter.  */
    reg->free += size;
    assert(reg->free <= reg->size - sizeof(struct lmm_region));

    /* Now find the location in that region's free list at which to add the node.  */
    for (prevnode = 0, nextnode = reg->nodes;
	 (nextnode != 0) && (nextnode < node);
	 prevnode = nextnode, nextnode = nextnode->next);

    /* Coalesce the new free chunk into the previous chunk if possible.  */
    if ((prevnode) && ((unsigned int) prevnode + prevnode->size >= (unsigned int) node))
    {
	assert((unsigned int) prevnode + prevnode->size == (unsigned int) node);

	/* Coalesce prevnode with nextnode if possible.  */
	if ((unsigned int) node + size >= (unsigned int) nextnode)
	{
	    assert((unsigned int) node + size == (unsigned int) nextnode);

	    prevnode->size += size + nextnode->size;
	    prevnode->next = nextnode->next;
	}
	else
	{
	    /* Not possible - just grow prevnode around newly freed memory.  */
	    prevnode->size += size;
	}
    }
    else
    {
	/* Insert the new node into the free list.  */
	if (prevnode)
	    prevnode->next = node;
	else
	    reg->nodes = node;

	/* Try coalescing the new node with the nextnode.  */
	if ((unsigned int) node + size >= (unsigned int) nextnode)
	{
	    node->size = size + nextnode->size;
	    node->next = nextnode->next;
	}
	else
	{
	    node->size = size;
	    node->next = nextnode;
	}
    }
}
