/*
 * LittleOS - http://come.to/littleos/
 * Copyright (C) 1998-2000 Lacroix Pascal <placr@hotmail.com>
 * All rights reserved.
 *
 * LittleOS is free software -- also known as "open source" -- you can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License, version 2, as published by the Free Software Foundation,
 * and found in the file `COPYING'.  This license basically allows free use,
 * modification, and redistribution, as long as the source to LittleOS and
 * any code linked against it is made freely available.  (The license in
 * `COPYING' is authoritative, not this interpretation of it.)
 *
 * LittleOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with LittleOS; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place #330, Boston, MA 02111-1307, USA, or look
 * at http://www.fsf.org/copyleft/gpl.html.
 *
 */

/* TODO: test these routines (they seem to work, but ...) */

#include <types.h>
#include <sys/paging.h>
#include <sys/kernel.h>
#include <malloc.h>		/* for malloc_lmm */
#include <sys/lmm.h>
#include <string.h>
#include <i386/selectors.h>

unsigned long *pd;

/* allocate a page of memory (4096 bytes) for the paging system
   (each page is also aligned to 4096 */
static void *
alloc_page(void)
{
    void *p = lmm_alloc_page(&malloc_lmm, 0);

    if (p)
	memset(p, 0, PAGE_SIZE);
    return p;
}

/* map page at linear to physical */
bool
vm_map_page(unsigned long linear, unsigned long physical)
{
    unsigned long *pd_entry, *pt_entry;

    linear = ALIGN_PAGE(linear);
    physical = ALIGN_PAGE(physical);

    pd_entry = pdir_find_pde(pd, linear);
    if (!(*pd_entry & PAGE_PRESENT))
    {
	/* we have to allocate a new page table entry */
	void *p = alloc_page();

	if (p == NULL)
	    return FALSE;

#ifdef DEBUG
	printk("vm_map_page: alloc a new page\n");
#endif

	*pd_entry = (unsigned long) p | PAGE_PRESENT | PAGE_RW;

#ifdef DEBUG
       printk("pd_entry= 0x%08X, *pd_entry= 0x%08x\n", pd_entry, *pd_entry);
#endif

    }

    pt_entry = ptab_find_pte(*pd_entry & 0xFFFFF000, linear);

    *pt_entry = physical | PAGE_PRESENT | PAGE_RW;

    return TRUE;
}

/* map a range of pages */
bool
vm_map_range(unsigned long linear, unsigned long physical, unsigned long size)
{
    long i, j, end;
    bool result = TRUE;

    end = i = ALIGN_PAGE(linear);
    j = ALIGN_PAGE(physical);
    size = ALIGN_PAGE(size);
    end += size;

    for (; i < end; i += PAGE_SIZE, j += PAGE_SIZE)
	result &= vm_map_page(i, j);

    return result;
}

/* not yet used
   static void reload_cr3(void)
   {
   asm volatile ("mov %%cr3, %%eax  \n\t"
   "mov %%cr3, %%eax  \n\t"
   "ljmp %0, $1f      \n\t": : "p" (KERNEL_CS): "eax");
   }
 */

/* change to mapping bits of a page (page corresponding to address linear) */
bool
vm_prot_page(unsigned long linear, long mappings)
{
    unsigned long *pt_entry, *pd_entry;

    linear = ALIGN_PAGE(linear);

    pd_entry = pdir_find_pde(pd, linear);
    if (!(*pd_entry & PAGE_PRESENT))
	return FALSE;

    pt_entry = ptab_find_pte(*pd_entry & 0xFFFFF000, linear);

    *pt_entry |= (mappings & 0xFFF);

    return TRUE;
}

/* unmap a page: suppress the mapping bits (can be "remapped"
   with vm_remap_page (see next function below) */
bool
vm_unmap_page(unsigned long linear)
{
    unsigned long *pt_entry, *pd_entry;

    linear = ALIGN_PAGE(linear);

    pd_entry = pdir_find_pde(pd, linear);
    if (!(*pd_entry & PAGE_PRESENT))
	return FALSE;

    pt_entry = ptab_find_pte(*pd_entry & 0xFFFFF000, linear);

    *pt_entry = (*pt_entry & 0xFFFFF000);

    return TRUE;
}

/* remap a page unmapped with vm_unmap_page */
bool
vm_remap_page(unsigned long linear, long mappings)
{
    unsigned long *pt_entry, *pd_entry;
    long old;

    linear = ALIGN_PAGE(linear);

    pd_entry = pdir_find_pde(pd, linear);

    pt_entry = ptab_find_pte(*pd_entry & 0xFFFFF000, linear);

    old = *pt_entry;
    *pt_entry = (*pt_entry & 0xFFFFF000) | PAGE_PRESENT | (mappings & 0xFFF);

    return TRUE;
}

static void
enable_paging(void)
{
    asm volatile ("movl %%eax, %%cr3"::
		  "a" ((long) pd));
    asm volatile ("movl %%cr0, %%eax     	\n\t"
		  "orl $0x80000000, %%eax \n\t"
		  "movl %%eax, %%cr0		\n\t"
		  "mov %%cr3, %%eax		\n\t"
		  "mov %%eax, %%cr3		\n\t"
		  "ljmp %0, $1f			\n\t"
		  "1:						\n\t"
		  ::"p" (KERNEL_CS):"%eax");
}

void
init_paging(void)
{
    pd = (unsigned long *) alloc_page();
    vm_map_range(0, 0, 4 * 1024 * 1024);	/* map 4MB (linear=physical) */
    vm_unmap_page(0);		/* first page is not accessible */

#ifdef DEBUG
    printk("enabling paging...");
#endif

    enable_paging();

#ifdef DEBUG
    printk(" OK\n");
#endif

}

