/* 
 * Mach Operating System
 * Copyright (c) 1992,1991,1990,1989 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

/*
 * Modifications for LittleOS by Pascal Lacroix (Jan 2000).
 */

#include <string.h>

char *
strcat(char *s, const char *add)
{
    register char *ret = s;

    while (*s)
	s++;

    while ((*s++ = *add++) != 0);

    return ret;
}

char *
strchr(const char *s, int c)
{
    while (1)
    {
	if (*s == c)
	    return (char *) s;
	if (*s == 0)
	    return 0;
	s++;
    }
}

int
strcmp(const char *s1, const char *s2)
{
    register unsigned int a, b;



    while ((a = *s1++), (b = *s2++), a && b)
    {
	if (a != b)
	    return (a - b);
    }

    return a - b;
}

char *
strcpy(char *to, const char *from)
{
    register char *ret = to;

    while ((*to++ = *from++) != 0);

    return ret;
}

int
strlen(const char *string)
{
    const char *ret = string;

    while (*string++);

    return string - 1 - ret;

}

char *
strncat(dst, src, n)
     char *dst;
     const char *src;
     register int n;
{
    if (n != 0)
    {
	register char *d = dst;
	register const char *s = src;

	while (*d != 0)
	    d++;
	do
	{
	    if ((*d = *s++) == 0)
		break;
	    d++;
	}
	while (--n != 0);
	*d = 0;
    }
    return (dst);
}

int
strncmp(const char *s1, const char *s2, int n)
{
    while (1)
    {
	if (n <= 0)
	    return 0;
	if (*s1 != *s2)
	    return *s1 - *s2;
	if (*s1 == 0)
	    return 0;

	s1++;
	s2++;
	n--;
    }
}

char *
strncpy(char *to, const char *from, int count)
{
    register char *ret = to;

    while (count > 0)
    {
	count--;
	if ((*to++ = *from++) == '\0')
	    break;
    }

    while (count > 0)
    {
	count--;
	*to++ = '\0';
    }

    return ret;
}

char *
strstr(const char *haystack, const char *needle)
{
    int hlen = strlen(haystack);
    int nlen = strlen(needle);

    while (hlen >= nlen)
    {
	if (!memcmp(haystack, needle, nlen))
	    return (char *) haystack;

	haystack++;
	hlen--;
    }
    return 0;
}
